@@ -11,10 +11,40 @@ Digital Loggers Web Power Switch management
 
               When run as a script this acts as a command
               line utility to manage the DLI Power switch.
+
+              This module has been tested against the following 
+              Digital Loggers Power network power switches:
+                WebPowerSwitch II
+                WebPowerSwitch III
+                WebPowerSwitch IV
+                WebPowerSwitch V
+                Ethernet Power Controller III
+              
  Author: Dwight Hubbard d@dhub.me
- Copyright: This module may be used for any use personal
-            or commercial as long as the author and copyright
-            notice are included in full.
+"""
+"""
+Copyright (c) 2009, Dwight Hubbard
+All rights reserved.
+
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions are met:
+
+1. Redistributions of source code must retain the above copyright notice,
+   this list of conditions and the following disclaimer.
+2. Redistributions in binary form must reproduce the above copyright notice,
+   this list of conditions and the following disclaimer in the documentation
+   and/or other materials provided with the distribution.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
 
 # Python built in modules
@@ -59,6 +89,7 @@ class powerswitch:
             self.cycletime=float(cycletime)
         else:
             self.cycletime=CONFIG['cycletime']
+        self._is_admin=True
     def load_configuration(self):
         """ Return a configuration dictionary """
         if os.path.isfile(CONFIG_FILE):
@@ -110,19 +141,39 @@ class powerswitch:
         #except timeout:
         #    return None
         return result
+    def determine_outlet(self,outlet=None):
+        """ Get the correct outlet number from the outlet passed in, this
+            allows specifying the outlet by the name and making sure the
+            returned outlet is an int 
+        """
+        outlets=self.statuslist()
+        if outlet and outlets and type(outlet) is str:
+            for plug in outlets:
+                if plug[1].strip() == outlet.strip():
+                    return int(plug[0])
+        return int(outlet)
+    def get_outlet_name(self,outlet=0):
+        """ Return the name of the outlet """
+        outlet=self.determine_outlet(outlet)
+        outlets=self.statuslist()
+        if outlets and outlet:
+            for plug in outlets:
+                if int(plug[0]) == outlet:
+                    return plug[1]
+        return 'Unknown'
     def off(self,outlet=0):
         """ Turn off a power to an outlet 
             False = Success
             True = Fail
         """
-        self.geturl(url= 'outlet?%d=OFF' % int(outlet))
+        self.geturl(url= 'outlet?%d=OFF' % self.determine_outlet(outlet))
         return self.status(outlet) != 'OFF'
     def on(self,outlet=0):
         """ Turn on power to an outlet 
             False = Success
             True = Fail
         """
-        self.geturl(url= 'outlet?%d=ON' % outlet)
+        self.geturl(url= 'outlet?%d=ON' % self.determine_outlet(outlet))
         return self.status(outlet) != 'ON'
     def cycle(self,outlet=0):
         """ Cycle power to an outlet 
@@ -147,7 +198,14 @@ class powerswitch:
         try:
             root=soup.findAll('td',text='1')[0].parent.parent.parent
         except IndexError:
-            return None
+            # Finding the root of the table with the outlet info failed
+            # try again assuming we're seeing the table for a user
+            # account insteaed of the admin account (tables are different)
+            try:
+              self._is_admin=False
+              root=soup.findAll('th',text='#')[0].parent.parent.parent
+            except IndexError:
+              return None
         for temp in root.findAll('tr'):
             columns=temp.findAll('td')
             if len(columns) == 5:
@@ -167,6 +225,7 @@ class powerswitch:
         return
     def status(self,outlet=1):
         """ Return the status of an outlet, returned value will be one of: ON, OFF, Unknown """
+        outlet=self.determine_outlet(outlet)
         outlets=self.statuslist()
         if outlets and outlet:
             for plug in outlets:
@@ -175,7 +234,7 @@ class powerswitch:
         return 'Unknown'
 
 if __name__ == "__main__":
-    usage = "usage: %prog [options] [status|on|off|cycle] [arg]"
+    usage = "usage: %prog [options] [status|on|off|cycle|get_outlet_name] [arg]"
     parser = optparse.OptionParser(usage=usage)
     parser.add_option('--hostname',dest='hostname',default=None,help="hostname/ip of the power switch (default %default)")
     parser.add_option('--timeout',dest='timeout',default=None,help="Timeout for value for power switch communication (default %default)")
@@ -183,6 +242,7 @@ if __name__ == "__main__":
     parser.add_option('--user',    dest='user',    default=None        ,help="userid to connect with (default %default)"         )
     parser.add_option('--password',dest='password',default=None         ,help="password (default %default)"                       )
     parser.add_option('--save_settings',dest='save_settings',default=False,action='store_true',help='Save the settings to the configuration file')
+    parser.add_option("--quiet",dest="quiet",default=False,action="store_true",help="Be quiet, don't print error messages only return error return codes (default False)")
     (options, args) = parser.parse_args()
 
     switch=powerswitch(userid=options.user,password=options.password,hostname=options.hostname,timeout=options.timeout,cycletime=options.cycletime)
@@ -191,12 +251,21 @@ if __name__ == "__main__":
     if len(args):
         if len(args) == 2:
             if args[0].lower() in ['on','poweron']:
-                sys.exit(switch.on(int(args[1])))
-            if args[0].lower() in ['off','poweroff']:
-                sys.exit(switch.off(int(args[1])))
-            if args[0].lower() in ['cycle']:
-                sys.exit(switch.cycle(int(args[1])))
-            if args[0].lower() in ['status']:
-                print(switch.status(int(args[1])))
+                rc=switch.on(args[1])
+                if rc and not options.quiet:
+                  print >> sys.stderr,"Power on operation failed"
+                sys.exit(rc)
+            elif args[0].lower() in ['off','poweroff']:
+                rc=switch.off(args[1])
+                if rc and not options.quiet:
+                  print >> sys.stderr,"Power off operation failed"
+            elif args[0].lower() in ['cycle']:
+                sys.exit(switch.cycle(args[1]))
+            elif args[0].lower() in ['status']:
+                print(switch.status(args[1]))
+            elif args[0].lower() in ['get_name','getname','get_outlet_name','getoutletname']:
+                print(switch.get_outlet_name(args[1]))
+            else:
+                print("Unknown argument %s" % args[0])
     else:
         switch.printstatus()
